"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseObjectExpression = void 0;
const cssTransform_1 = require("./cssTransform");
function parseObjectExpression(object, retArray, hashable) {
    return object.properties.forEach((p) => handleProperties(p, retArray, hashable));
}
exports.parseObjectExpression = parseObjectExpression;
function handleSpread(spread, retArray, hashable) {
    const argument = spread.argument;
    if (argument.type === "ObjectExpression") {
        return parseObjectExpression(argument, retArray, hashable);
    }
    throw Error(`Cannot parse ${spread.type}. Catom compiler only accepts compile time constant values`);
}
function handleProperties(propertyOrSpread, retArray, hashable) {
    if (propertyOrSpread.type === "SpreadElement") {
        return handleSpread(propertyOrSpread, retArray, hashable);
    }
    let { key, value } = propertyOrSpread;
    if (value.type === "TSAsExpression")
        value = value.expression;
    let keyName;
    if (key.type === "StringLiteral") {
        keyName = key.value;
    }
    else if (key.type === "Identifier") {
        keyName = key.name;
    }
    else {
        return throwErr();
    }
    const isMedia = keyName === "media";
    const isPseudo = keyName === "pseudo";
    const canAcceptObjectLiteralInValue = isMedia || isPseudo;
    if (value.type === "StringLiteral" || value.type === "NumericLiteral") {
        if (canAcceptObjectLiteralInValue)
            throwErr("Need an object literal for media query or pseudo selector");
        return retArray.push((0, cssTransform_1.createValueHash)(keyName, value.value, hashable));
    }
    if (canAcceptObjectLiteralInValue && value.type === "ObjectExpression") {
        return value.properties.forEach((prop) => handleMediaOrPseudoProperties(prop, retArray, isMedia, isPseudo));
    }
    throwErr();
}
function handleMediaOrPseudoProperties(property, retArray, isMedia, isPseudo) {
    if (property.type === "ObjectProperty") {
        const { key, value } = property;
        let keyName;
        if (key.type === "StringLiteral")
            keyName = key.value;
        else if (key.type === "Identifier")
            keyName = key.name;
        if (value.type === "ObjectExpression") {
            return parseObjectExpression(value, retArray, (isMedia && { media: keyName }) || (isPseudo && { pseudo: keyName }));
        }
    }
    throwErr();
}
function throwErr(err) {
    throw TypeError(err || "Catom only accepts literals and compile time constant values");
}
